/*
 * BallWorld.java
 *
 * Created on September 14, 2005, 10:07 PM
 *
 * BallWorld for ENGR2510, fall 2005 at Olin, part of the CS101 project
 * (c)2005 Olin College
 */

package ballworld.ui;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import ballworld.*;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;


/**
 * The BallWorld window, including the menu bar and ball buttons.
 *
 * @author Mike Wessler
 */
public class BallWorld extends JFrame {
    /** the drawing area */
    private WorldCanvas wc;
    
    /** the panel of ball creation buttons */
    private Container buttonpanel;
        
    /** whether the action is paused or not */
    private boolean paused = false;

    /*--- Menu items ---*/
    private Action addClassAction = new AbstractAction("Add class...") {
        public void actionPerformed(ActionEvent evt) {
            doAddClass();
        }
    };
    private Action resetAction = new AbstractAction("Reset") {
        public void actionPerformed(ActionEvent evt) {
            doReset();
        }
    };
    private Action quitAction = new AbstractAction("Quit") {
        public void actionPerformed(ActionEvent evt) {
            System.exit(0);
        }
    };
    private Action pauseAction = new AbstractAction("Pause") {
        public void actionPerformed(ActionEvent evt) {
            doPause();
        }
    };
    /*--- End menu items ---*/

    /**
     * A button action that brings forth a new ball of a particular type
     */
    class AddBallAction extends AbstractAction {
        private Class clz;

        /**
         * Creates a new AddBallAction.
         * @param className the full name of the ball class.  It must implement
         * the Ball interface.
         * @throws IllegalArgumentException if the class doesn't implement the
         * Ball interface.
         */
        public AddBallAction(String className) throws IllegalArgumentException {
            super(className);
            try {
                Class c = Class.forName(className);
                if (!Ball.class.isAssignableFrom(c)) {
                    throw new IllegalArgumentException(className+" is not a Ball");
                }
                this.clz = c;
            } catch (ClassNotFoundException cnfe) {
                throw new IllegalArgumentException("Class "+className+
                                                   " not found");
            }
        }
        
        /**
         * Creates a new ball and adds it to thw world when the action is performed.
         */
        public void actionPerformed(ActionEvent evt) {
            try {
                Ball b = (Ball)clz.newInstance();
                wc.addBall(b);
            }  catch (Exception ex) {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(wc, "Couldn't create new "+
                        clz.getCanonicalName());
            }
        }
    }
    
    /**
     * Creates a new instance of BallWorld, builds the UI, and displays the
     * resulting window.
     * @param args an array of Strings that are the full names of classes for
     * initial ball buttons.
     */
    public BallWorld(String args[]) {
        super("BallWorld");
        createMenu();
        wc = new WorldCanvas();
        JPanel buttonframe = new JPanel();
        buttonpanel = Box.createVerticalBox();
        buttonframe.add(buttonpanel);
        buttonframe.setBorder(
                BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        getContentPane().add(wc);
        getContentPane().add(buttonframe, BorderLayout.WEST);
        for (String arg : args) {
            try {
                addClass(arg);
            } catch (IllegalArgumentException iae) {
                System.out.println(iae.getMessage());
            }
        }
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        pack();
        setVisible(true);
    }

    /**
     * Creates the menu bar.
     */
    private void createMenu() {
        JMenuBar mb = new JMenuBar();
        JMenu file = new JMenu("File");
        file.add(addClassAction);
        file.add(pauseAction);
        file.add(resetAction);
        file.addSeparator();
        file.add(quitAction);
        mb.add(file);
        setJMenuBar(mb);
    }
    
    /**
     * Adds a single class to the button panel.
     * @param theClass the full name of a class that implements the Ball
     * interface
     * @throws IllegalArgumentException if the class doesn't implement the
     * Ball interface.
     */
    private void addClass(String theClass) throws IllegalArgumentException {
        Action action = new AddBallAction(theClass);
        JButton button = new JButton(action);
        buttonpanel.add(Box.createVerticalStrut(4));
        buttonpanel.add(button);
    }

    /**
     * Prompts the user for the name of a class, and adds that class to the
     * button panel.
     */
    private void doAddClass() {
        String name = JOptionPane.showInputDialog(wc, "Name of class to find?", "");
        if (name != null) {
            try {
                addClass(name);
                // re-layout the button panel
                getContentPane().validate();
            } catch (IllegalArgumentException iae) {
                // tell the user we couldn't create the button
                JOptionPane.showMessageDialog(wc, iae.getMessage(),
                        "Couldn't create "+name, JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * Resets the board
     */
    private void doReset() {
        wc.clear();
    }
    
    /**
     * Toggles the paused state of the action
     */
    private void doPause() {
        paused = !paused;
        if (paused) {
            pauseAction.putValue(pauseAction.NAME, "Resume");
        } else {
            pauseAction.putValue(pauseAction.NAME, "Pause");
        }
        wc.setPaused(paused);
    }
    
    /**
     * All java programs start in a method with the following signature.
     * @param args an array of the full class names of classes that should be
     * initially displayed in the button panel.  The classes must implement
     * the Ball interface.
     */
    public static void main(String args[]) {
        new BallWorld(args);
    }
}
